# SPDX-License-Identifier: WTFPL
# SPDX-FileCopyrightText: 2023 Anna <cyber@sysrq.in>
# No warranty

""" Metadata routines """

import logging
import textwrap
import xml.etree.ElementTree as ET
from pathlib import Path
from typing import Callable

from gentle.metadata.types import Person, RemoteID, Upstream

logger = logging.getLogger("metadata")


class MetadataXML:
    """ Modify :file:`metadata.xml` files """

    def __init__(self, xmlfile: Path, parser: Callable[[Path], Upstream]):
        """
        :param xmlfile: Path to the :file:`metadata.xml` file
        :param upstream: Pre-parsed :class:`Upstream` object
        """
        self.xmlfile: Path = xmlfile
        self.xml: ET.ElementTree = ET.parse(self.xmlfile)

        self.upstream: Upstream = parser(xmlfile)

    def dump(self) -> None:
        """ Write :file:`metadata.xml` file """
        logger.info("Writing metadata.xml")
        ET.indent(self.xml, space="\t", level=0)
        with open(self.xmlfile, "w") as file:
            file.write(textwrap.dedent("""\
                <?xml version="1.0" encoding="UTF-8"?>
                <!DOCTYPE pkgmetadata SYSTEM "https://www.gentoo.org/dtd/metadata.dtd">
            """))
            self.xml.write(file, encoding="unicode")
            file.write("\n")

    def dumps(self) -> str:
        """ Convert the object to text """
        ET.indent(self.xml, space="\t", level=0)
        return ET.tostring(self.xml.getroot(), encoding="unicode")

    def add_upstream_maintainer(self, person: Person) -> None:
        """ Add a person to the list of upstream maintainers """
        if person in self.upstream.maintainers:
            return

        logger.info("Adding upstream maintainer: %s", person)
        self.upstream.maintainers.append(person)
        upstream = self._make_upstream_element()
        upstream.append(person.to_xml())

    def add_upstream_remote_id(self, remote_id: RemoteID) -> None:
        """ Add an item to the list of remote ids """
        if remote_id in self.upstream.remote_ids:
            return

        logger.info("Adding remote id: %s", remote_id)
        self.upstream.remote_ids.append(remote_id)
        upstream = self._make_upstream_element()
        upstream.append(remote_id.to_xml())

    def set_upstream_bugs_to(self, url: str) -> None:
        """ Set upstream bugs-to URL """
        if self.upstream.bugs_to:
            return

        logger.info("Setting upstream bug tracker to %s", url)
        self.upstream.bugs_to = url

        upstream = self._make_upstream_element()
        bugs_to = ET.SubElement(upstream, "bugs-to")
        bugs_to.text = url

    def set_upstream_changelog(self, url: str) -> None:
        """ Set upstream changelog URL """
        if self.upstream.changelog:
            return

        logger.info("Setting upstream changelog to %s", url)
        self.upstream.changelog = url

        upstream = self._make_upstream_element()
        changelog = ET.SubElement(upstream, "changelog")
        changelog.text = url

    def set_upstream_doc(self, url: str) -> None:
        """ Set upstream documentation URL """
        if self.upstream.doc:
            return

        logger.info("Setting upstream documentation to %s", url)
        self.upstream.doc = url

        upstream = self._make_upstream_element()
        doc = ET.SubElement(upstream, "doc")
        doc.text = url

    def _make_upstream_element(self) -> ET.Element:
        if (upstream := self.xml.find("upstream")) is None:
            pkgmetadata = self.xml.getroot()
            upstream = ET.SubElement(pkgmetadata, "upstream")
        return upstream



# SPDX-License-Identifier: WTFPL
# SPDX-FileCopyrightText: 2023 Anna <cyber@sysrq.in>
# No warranty

""" Types for working with Gentoo package metadata """

import xml.etree.ElementTree as ET
from dataclasses import dataclass, field


@dataclass
class Person:
    """ Representation of a person"""

    name: str = field(default="", compare=False)
    email: str = ""

    def to_xml(self, attrib: dict | None = None) -> ET.Element:
        """
        :param attrib: attributes for the ``<maintainer>`` tag
        :return: :file:`metadata.xml` respresentation of a person
        """
        result = ET.Element("maintainer", attrib=attrib or {})
        if self.name:
            name_elem = ET.SubElement(result, "name")
            name_elem.text = self.name
        if self.email:
            email_elem = ET.SubElement(result, "email")
            email_elem.text = self.email

        return result


@dataclass
class RemoteID:
    """ Representation of a remote ID """

    attr: str
    value: str

    def to_xml(self) -> ET.Element:
        """
        :return: :file:`metadata.xml` respresentation of a remote id
        """
        remote_elem = ET.Element("remote-id", type=self.attr)
        remote_elem.text = self.value
        return remote_elem


@dataclass
class Upstream:
    """ Representation of upstream metadata """

    maintainers: list[Person] = field(default_factory=list)
    changelog: str | None = None
    doc: str | None = None
    bugs_to: str | None = None
    remote_ids: list[RemoteID] = field(default_factory=list)



# SPDX-License-Identifier: WTFPL
# SPDX-FileCopyrightText: 2023 Anna <cyber@sysrq.in>
# No warranty

""" Common data for Python generators """

BUG_TRACKER_LABELS = ["bug tracker", "bugtracker", "bugs", "issues"]
CHANGELOG_LABELS = ["changelog", "changes"]
DOCS_LABELS = ["doc", "docs", "documentation"]
HOME_REPO_LABELS = ["source", "repo", "repository", "home", "homepage"]
